from django.contrib import admin

from micro_framework.jwt_auth.models import Policy

# Register your models here.

class PolicyAdmin(admin.ModelAdmin):
    search_fields = ['name']

admin.site.register(Policy, PolicyAdmin)



from micro_framework.settings import api_settings

def get_real_ip_from_header_meta(request_meta):
    """
    Getting real ip from request meta
    """
    for key_header in api_settings.HEADER_IP_REMOTE:
        real_ip = request_meta.get(key_header, None)
        if real_ip:
            return real_ip

def get_user_agent_from_header_meta(request_meta):
    """
    Getting user agent from request meta
    """
    return request_meta.get('HTTP_USER_AGENT', '')

def check_client_ip_and_token_ip(request):
    """
    Checking client ip with token ip
    """
    if request.auth is None:
        return False

    if getattr(request.auth, 'payload') is None:
        return False

    token_real_ip = request.auth.payload.get(api_settings.USER_IP_CLAIM, '')
    real_ip = get_real_ip_from_header_meta(request.META)
    if token_real_ip != real_ip:
        return False
    return True



from datetime import timedelta

from django.conf import settings
from django.test.signals import setting_changed
from django.utils.translation import ugettext_lazy as _
from rest_framework.settings import APISettings as _APISettings

from micro_framework.jwt_auth.utils import format_lazy

USER_SETTINGS = getattr(settings, 'MICRO_FRAMEWORK', None)

DEFAULTS = {
    'SERVICE_NAME': 'Default',
    'ACCESS_TOKEN_LIFETIME': timedelta(minutes=5),
    'REFRESH_TOKEN_LIFETIME': timedelta(days=1),
    'ROTATE_REFRESH_TOKENS': False,
    'BLACKLIST_AFTER_ROTATION': True,

    'ALGORITHM': 'HS256',
    'SIGNING_KEY': settings.SECRET_KEY,
    'VERIFYING_KEY': None,
    'AUDIENCE': None,
    'ISSUER': None,

    'AUTH_HEADER_TYPES': ('Bearer',),
    'USER_ID_FIELD': 'id',
    'USER_ID_CLAIM': 'user_id',

    'AUTH_TOKEN_CLASSES': ('micro_framework.jwt_auth.tokens.AccessToken',),
    'TOKEN_TYPE_CLAIM': 'token_type',

    'JTI_CLAIM': 'jti',

    # Custom payload Token
    'ADD_USER_IP_CLAIM': False,
    'USER_IP_CLAIM': 'user_ip',

    'ADD_USER_AGENT_CLAIM': False,
    'USER_AGENT_CLAIM': 'user_agent',

    'VALIDATE_SOURCE_IP': False,
    'HEADER_IP_REMOTE': ['HTTP_X_FORWARDED_FOR', 'REMOTE_ADDR'],
    

    # Redis backend
    'REDIS_ENABLE': False,
    'REDIS_HOST': 'localhost',
    'REDIS_PORT': 6379,
    'REDIS_PASSWORD': None,
    'REDIS_DB': 0,
    'REDIS_EXPIRE_TOKEN': False,
}

IMPORT_STRINGS = (
    'AUTH_TOKEN_CLASSES',
)

REMOVED_SETTINGS = (
    'AUTH_HEADER_TYPE',
    'AUTH_TOKEN_CLASS',
    'SECRET_KEY',
    'TOKEN_BACKEND_CLASS',
)


class APISettings(_APISettings):  # pragma: no cover
    def __check_user_settings(self, user_settings):
        SETTINGS_DOC = 'https://github.com/lacoski/django-micro-framework'

        for setting in REMOVED_SETTINGS:
            if setting in user_settings:
                raise RuntimeError(format_lazy(
                    _("The '{}' setting has been removed. Please refer to '{}' for available settings."),
                    setting, SETTINGS_DOC,
                ))

        return user_settings


api_settings = APISettings(USER_SETTINGS, DEFAULTS, IMPORT_STRINGS)


def reload_api_settings(*args, **kwargs):  # pragma: no cover
    global api_settings

    setting, value = kwargs['setting'], kwargs['value']

    if setting == 'MICRO_FRAMEWORK':
        api_settings = APISettings(value, DEFAULTS, IMPORT_STRINGS)


setting_changed.connect(reload_api_settings)



from django.core.management.base import BaseCommand, CommandError

from micro_framework.jwt_auth.models import Policy


class Command(BaseCommand):
    help = 'Delete Policy for service'

    def delete_service_policy(self, policy_name):
        Policy.objects.get(name=policy_name).delete()

    def add_arguments(self, parser):
        parser.add_argument('policyname', type=str)

    def handle(self, *args, **kwargs):
        policy_name = kwargs['policyname']

        try:
            Policy.objects.get(name=policy_name).delete()
        except Exception as ex:
            raise CommandError(
                'Cannot delete the Policy ({}) '.format(
                    policy_name)
            )
        self.stdout.write(
            'Delete Policy ({})'.format(policy_name))



import json


def data_to_json(data):
    return json.dumps(data)

def byte_to_string(data):
    return data.decode("utf8")

def redis_to_data(data):
    if isinstance(data, bytes):
        data = byte_to_string(data)
    return json.loads(data)



from rest_framework import generics, status
from rest_framework.response import Response

from micro_framework.jwt_auth import serializers
from micro_framework.jwt_auth.authentication import AUTH_HEADER_TYPES
from micro_framework.jwt_auth.exceptions import InvalidToken, TokenError


class TokenViewBase(generics.GenericAPIView):
    permission_classes = ()
    authentication_classes = ()

    serializer_class = None

    www_authenticate_realm = 'api'

    def get_authenticate_header(self, request):
        return '{0} realm="{1}"'.format(
            AUTH_HEADER_TYPES[0],
            self.www_authenticate_realm,
        )

    def post(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)

        try:
            serializer.is_valid(raise_exception=True)
        except TokenError as e:
            raise InvalidToken(e.args[0])

        return Response(serializer.validated_data, status=status.HTTP_200_OK)


class TokenObtainPairView(TokenViewBase):
    """
    Takes a set of user credentials and returns an access and refresh JSON web
    token pair to prove the authentication of those credentials.
    """
    serializer_class = serializers.TokenObtainPairSerializer


token_obtain_pair = TokenObtainPairView.as_view()


class TokenRefreshView(TokenViewBase):
    """
    Takes a refresh type JSON web token and returns an access type JSON web
    token if the refresh token is valid.
    """
    serializer_class = serializers.TokenRefreshSerializer


token_refresh = TokenRefreshView.as_view()


class TokenVerifyView(TokenViewBase):
    """
    Takes a token and indicates if it is valid.  This view provides no
    information about a token's fitness for a particular use.
    """
    serializer_class = serializers.TokenVerifySerializer


token_verify = TokenVerifyView.as_view()


class RedisTokenObtainPairView(TokenViewBase):
    serializer_class = serializers.TokenObtainPairSerializer
